@@ -27,6 +27,16 @@ preloader={
 						'desc':f'return a selected drug text',
 						'exec':drug_text
 					},
+	f'{uuid1()}':{
+						'cmds':['temp logger','temp log','tmplg'],
+						'desc':f'log a temperature',
+						'exec':Templogger
+					},
+	f'{uuid1()}':{
+						'cmds':['mulefraud','mule fraud','mule-fraud','tax fraud mulisha','tax fraud 11.8.2025','consumer fraud 11.8.2025'],
+						'desc':f'see what tax fraud values might look like',
+						'exec':TaxMuleFraud
+					},
 	f'{uuid1()}':{
 						'cmds':['golden-ratio','gldn rto',],
 						'desc':f'get the golden ration for a measurement',
