@@ -2111,3 +2111,286 @@ def BackroomLocation():
         fb['caseID']=f"{DELCHAR}{fb['caseID']}"
     locationString=f"{fb['moduleType']}{DELCHAR}{fb['moduleNumberRange']}{' '.join(db.stre(fb['caseID'])/3)}"
     return locationString
+
+def TaxMuleFraud():
+    msg=''
+    while True:
+        try:
+            fields={
+                'Price':{
+                'type':'dec.dec',
+                'default':decc(2.50,)
+                },
+                'Legally Applied Tax':{
+                'type':'dec.dec',
+                'default':decc(0.01)
+                },
+                'Mule Tax Rate (Charged as Despite being legally the other)':{
+                'type':'dec.dec',
+                'default':decc(0.063)
+                }
+            }
+            fd=FormBuilder(data=fields,passThruText=msg)
+            if fd is None:
+                return
+                
+            prc=fd['Price']
+            ltx=fd['Legally Applied Tax']
+            iltx=fd['Mule Tax Rate (Charged as Despite being legally the other)']
+            print(BooleanAnswers.FraudAlert)
+            totals={
+            f'legally_taxed_tax[(Price({prc})*legal_tax({ltx})]':fd['Price']*fd['Legally Applied Tax'],
+            f'legally_taxed_price[Price({prc})+(Price({prc})*legal_tax({ltx})]':(fd['Price']*fd['Legally Applied Tax'])+fd['Price'],
+
+            f'illegally_taxed_tax[Price({prc})*Illegal Tax({iltx})]':fd['Price']*fd['Mule Tax Rate (Charged as Despite being legally the other)'],
+            f'illegally_taxed_price(What was charged to the customer)[Price({prc})+(Price({prc})*Illegal_tax({iltx}))]':(fd['Price']*fd['Mule Tax Rate (Charged as Despite being legally the other)'])+fd['Price'],
+
+            f'fraud_profit_tax_only[Price({prc})*(illegal_tax({iltx})-legal_tax({ltx})]':fd['Price']*(fd['Mule Tax Rate (Charged as Despite being legally the other)']-fd['Legally Applied Tax']),
+            f'fraud_profit_as_price[Price({prc})+(Price({prc})*(illegal_tax({iltx})-legal_tax({ltx}))]':(fd['Price']*(fd['Mule Tax Rate (Charged as Despite being legally the other)']-fd['Legally Applied Tax']))+fd['Price'],
+            }
+            cta=len(totals)
+            htext='\n'.join([std_colorize(f'{i} = {Fore.light_green}{totals[i]}',num,cta) for num,i in enumerate(totals)])
+            while True:
+                which=Control(func=FormBuilderMkText,ptext=f"{htext}\nwhich index to return?",helpText=htext,data="integer")
+                if which in [None,'NaN']:
+                    return 
+                elif which in range(0,len(totals)):
+                    return totals[[i for i in totals.keys()][which]]
+                else:
+                    continue
+
+            value=None
+
+            return value
+        except Exception as e:
+            print(e)
+
+class Templogger:
+    def fix_table(self):
+        TemplLog.__table__.drop(ENGINE)
+        TemplLog.metadata.create_all(ENGINE)
+        print("Done!")
+
+
+    def newLog(self,temp=None):
+        with Session(ENGINE) as session:
+            excludes=['templogid',]
+            provided=False
+            print(temp)
+            if temp is None:
+                temp=TemplLog()
+            else:
+                provided=True
+
+            if not provided:
+                session.add(temp)
+                session.commit()
+                session.refresh(temp)
+            else:
+                temp=session.query(TemplLog).filter(TemplLog.templogid==temp.templogid).first()
+            print(temp)
+
+            fields={str(i.name):{'default':getattr(temp,i.name),'type':str(i.type).lower()} for i in temp.__table__.columns if i.name not in excludes}
+            fb=FormBuilder(data=fields,passThruText="if you see #UNUS, then only state that info if it is needed!")
+            if fb in [None,]:
+                session.delete(temp)
+                session.commit()
+                return
+            for k in fb:
+                setattr(temp,k,fb[k])
+            temp.dtoe=datetime.now()
+            session.commit()
+            session.refresh(temp)
+            print(temp)
+            return temp
+
+    def short_view(self,data:list,printToScreen=True,num=None):
+        xtext=[]
+        ct=len(data)
+        ii=''
+        for xnum,i in enumerate(data):
+            processed_templerature_string=pint.Quantity(i.templerature_value,i.templerature_unit)
+            msg=f"""TemplLog ->\n {ii}    
+{Fore.light_red}[{Fore.cyan}templogid{Fore.light_red}] {Fore.light_green}{i.templogid}
+{Fore.light_red}[{Fore.cyan}dtoe{Fore.light_red}] {Fore.light_green}{i.dtoe}
+{Fore.light_red}[{Fore.cyan}Logged{Fore.light_yellow} Temperature{Fore.light_red}] {Fore.light_green}{processed_templerature_string.to('degC')} {Fore.green_yellow}or {processed_templerature_string.to('degF')} {Fore.light_magenta}or {processed_templerature_string.to('degK')}
+{Fore.light_red}[{Fore.cyan}Location{Fore.light_red}] {Fore.light_green}{i.Location}
+{Fore.light_red}[{Fore.cyan}EmployeeIDorNAME{Fore.light_red}] {Fore.light_green}{i.EmployeeIDorNAME}
+{Fore.light_red}[{Fore.cyan}note{Fore.light_red}] {Fore.light_green}{i.note}{Style.reset}
+            """
+            if not num:
+                m=std_colorize(msg,xnum,ct)
+            else:
+                m=std_colorize(msg,num,ct)
+            xtext.append(m)
+            if printToScreen:            
+                print(m)
+        return '\n'.join(xtext)
+
+    def long_view(self,data:list,printToScreen=True,num=None):
+        xtext=[]
+        ct=len(data)
+        for numx,i in enumerate(data):
+            if not num:
+                m=std_colorize(i,numx,ct)
+            else:
+                m=std_colorize(i,num,ct)
+            xtext.append(m)
+            if printToScreen:
+                print(m)
+        return '\n'.join(xtext)
+
+
+    def edit_id(self,item):
+        try:
+            with Session(ENGINE) as session:
+                log=session.query(TemplLog).filter(TemplLog.templogid==item.templogid).first()
+                return self.newLog(temp=log)
+        except Exception as e:
+            print(e)
+            return item
+        pass
+
+    def delete_id(self,item):
+        try:
+            with Session(ENGINE) as session:
+                log=session.query(TemplLog).filter(TemplLog.templogid==item.templogid).first()
+                session.delete(log)
+                session.commit()
+        except Exception as e:
+            print(e)
+            return item
+        pass
+
+    def search_and_menu(self,menu=False,short=True):
+        with Session(ENGINE) as session:
+            filt=[
+
+            ]
+            core={
+                'limit':{
+                'default':10,
+                'type':"integer"
+                },
+                'offset':{
+                'default':0,
+                'type':'integer,'
+                }
+            }
+            fd=FormBuilder(data=core,passThruText="limit your results.")
+            if fd is None:
+                return
+            query=session.query(TemplLog).filter(or_(*filt))
+            orderedQuery=orderQuery(query,TemplLog.dtoe)
+            limited=limitOffset(query,limit=fd['limit'],offset=fd['offset'])
+            results=limited.all()
+            if short:
+                htext=self.short_view(results,False)
+            else:
+                htext=self.long_view(results,False)
+
+            if menu:
+                #next update
+                cta=len(results)
+                gotoNext=None
+                for num,log in enumerate(results):
+                    if log == None:
+                        continue
+                    while True:
+                        if log is None:
+                            gotoNext=True
+                            break
+                        
+                        cmds=['edit=ed/edit/e','delete=rm/del/delete/remove']
+                        try:
+                            if short:
+                                l=self.short_view([log,],False,num)
+                            else:
+                                l=self.long_view([log,],False,num)
+                            doWhat=Control(func=FormBuilderMkText,ptext=f"{l}\n[{cmds}]?",helpText=f"{l}\n{cmds}",data="string")
+                            if doWhat in ['NaN',None]:
+                                return
+                            elif doWhat.lower() in ['d','']:
+                                gotoNext=True
+                                break
+                            elif doWhat.lower() in [i.lower() for i in 'edit=ed/edit/e'.split('=')[-1].split("/")]:
+                                log=self.edit_id(log)
+
+                                continue
+                            elif doWhat.lower() in [i.lower() for i in 'delete=rm/del/delete/remove'.split('=')[-1].split("/")]:
+                                log=self.delete_id(log)
+                                continue
+                            else:
+                                gotoNext=True
+                                break
+                            if gotoNext:
+                                gotoNext=False
+                                break
+                        except Exception as e:
+                            print(e)
+                            gotoNext=True
+                            break
+            else:
+                print(htext)
+
+
+
+    def __init__(self):
+        cmds={
+            str(uuid1()):{
+            'cmds':['fixtable','fx tbl'],
+            'exec':self.fix_table,
+            'desc':"regenerate tables; a complete clear!!!"
+            },
+            str(uuid1()):{
+            'cmds':['search','sch'],
+            'exec':self.search_and_menu,
+            'desc':"search and print - LONG!!!"
+            },
+            str(uuid1()):{
+            'cmds':['search','sch'],
+            'exec':lambda self=self: self.search_and_menu(menu=True,short=False),
+            'desc':"search and use menu - LONG!!!"
+            },
+            str(uuid1()):{
+            'cmds':['search','sch'],
+            'exec':lambda self=self: self.search_and_menu(short=True),
+            'desc':"search and print - Short!!!"
+            },
+            str(uuid1()):{
+            'cmds':['search','sch'],
+            'exec':lambda self=self: self.search_and_menu(menu=True,short=True),
+            'desc':"search and use menu - Short!!!"
+            },
+            
+            str(uuid1()):{
+            'cmds':['new log','new','new temp log','ntl'],
+            'exec':self.newLog,
+            'desc':"create a new log"
+            }
+        }
+        htext=[]
+        cta=len(cmds.keys())
+        for num,i in enumerate(cmds):
+            if str(num) not in cmds[i]['cmds']:
+                cmds[i]['cmds'].append(str(num))
+            msg=f"{cmds[i]['cmds']} - {Fore.light_green}{cmds[i]['desc']}"
+            htext.append(std_colorize(msg,num,cta))
+        htext='\n'.join(htext)
+        while True:
+            doWhat=Control(func=FormBuilderMkText,ptext=f"{Fore.orange_red_1}Templogger {Fore.light_green}Exec:",helpText=htext,data="string")
+            if doWhat in [None,"NaN"]:
+                return None
+            elif doWhat in ['d','']:
+                print(htext)
+                continue
+            for c in cmds:
+                if doWhat.lower() in [i.lower() for i in cmds[c]['cmds']]:
+                    if callable(cmds[c]['exec']):
+                        try:
+                            cmds[c]['exec']()
+                        except Exception as e:
+                            print(e)
+                            break
+                    else:
+                        print(cmds[c],"!Callable()")
